@@ -107,6 +107,7 @@ pub fn correct_cable_lengths(
     );
     correction_progress.set_message("cable corrections");
 
+    // TODO: his whole thing is pretty inelegant, needs a re-do.
     jones_array
         .axis_iter_mut(Axis(2))
         .into_par_iter()
@@ -321,7 +322,6 @@ pub fn correct_geometry(
 /// # Assumptions
 /// - the digital gains are provided in [`mwalib::Rfinput.digital_gains`] in the same order as the
 ///   coarse channel indices (increasing sky frequency)
-/// - the gains for the x rfinput are the same as the y rfinput for each antenna
 ///
 /// # Errors
 /// - Will throw BadArrayShape if:
@@ -331,12 +331,10 @@ pub fn correct_digital_gains(
     context: &CorrelatorContext,
     jones_array: &mut Array3<Jones<f32>>,
     sel_coarse_chan_range: &Range<usize>,
-    // The tile index pairs for each selected baseline
     sel_baselines: &[(usize, usize)],
 ) -> Result<(), BirliError> {
     let num_fine_chans_per_coarse = context.metafits_context.num_corr_fine_chans_per_coarse;
 
-    // TODO: proper error handling
     let vis_dims = jones_array.dim();
     if vis_dims.1 != sel_coarse_chan_range.len() * num_fine_chans_per_coarse {
         return Err(BirliError::BadArrayShape {
@@ -369,13 +367,6 @@ pub fn correct_digital_gains(
         |(ant_idx, coarse_chan_idx)| {
             let ant = context.metafits_context.antennas[ant_idx].clone();
             let mwalib_coarse_chan_idx = sel_coarse_chan_range.start + coarse_chan_idx;
-            // assert_eq!(
-            //     ant.rfinput_x.digital_gains[mwalib_coarse_chan_idx],
-            //     ant.rfinput_y.digital_gains[mwalib_coarse_chan_idx],
-            //     "x and y digital gains for antenna {:?} are not equal for coarse channel {}",
-            //     ant,
-            //     mwalib_coarse_chan_idx
-            // );
             (
                 ant.rfinput_x.digital_gains[mwalib_coarse_chan_idx],
                 ant.rfinput_y.digital_gains[mwalib_coarse_chan_idx],
@@ -448,22 +439,224 @@ fn _correct_digital_gains(
     Ok(())
 }
 
+/// Correct for coarse pfb bandpass shape in each coarse channel by scaling passband_gains to
+/// fit the coarse band.
+///
+/// # Arguments
+///
+/// - `jones_array` - The array of Jones matrices to be corrected, [timestep][channel][baseleine].
+/// - `weight_array` - The array of weights to be corrected, same dimensions are `jones_array`.
+/// - `passband_gains` - a slice of gains to be applied to each coarse channel.
+/// - `num_fine_chans_per_coarse` - The number of fine channels in each coarse.
+/// - `mwax` - Whether to emulate the MWAX correlator (true) or legacy (false)
+///
+/// # Errors
+///
+/// Will throw `BirliError::BadArrayShape` if:
+/// - `num_fine_chans_per_coarse` is zero
+/// - The length of the channel axis in `jones_array` is not a multiple of num_fine_chans_per_coarse.
+/// - `jones_array` and `weight_array` have different shapes.
+/// - The length of the coarse band gains is not a multiple of num_fine_chans_per_coarse, or vice versa.
+pub fn correct_coarse_passband_gains(
+    jones_array: &mut Array3<Jones<f32>>,
+    weight_array: &mut Array3<f32>,
+    passband_gains: &[f64],
+    num_fine_chans_per_coarse: usize,
+    mwax: bool,
+) -> Result<(), BirliError> {
+    // # TODO: rename correct_passband_gains
+    if num_fine_chans_per_coarse == 0 {
+        return Err(BirliError::BadArrayShape {
+            argument: "num_fine_chans_per_coarse".into(),
+            function: "correct_coarse_passband_gains".into(),
+            expected: "a number greater than zero".into(),
+            received: format!("{:?}", num_fine_chans_per_coarse),
+        });
+    }
+
+    if jones_array.dim().1 % num_fine_chans_per_coarse != 0 {
+        return Err(BirliError::BadArrayShape {
+            argument: "jones_array".into(),
+            function: "correct_coarse_passband_gains".into(),
+            expected: format!(
+                "(_, n, _), where n is a multiple of num_fine_chans_per_coarse={}",
+                num_fine_chans_per_coarse
+            ),
+            received: format!("{:?}", jones_array.dim()),
+        });
+    };
+
+    if weight_array.dim() != jones_array.dim() {
+        return Err(BirliError::BadArrayShape {
+            argument: "weight_array".into(),
+            function: "correct_coarse_passband_gains".into(),
+            expected: format!("same as jones_array.dim()={:?}", jones_array.dim()),
+            received: format!("{:?}", weight_array.dim()),
+        });
+    };
+
+    let fscrunch = if passband_gains.len() % num_fine_chans_per_coarse == 0 {
+        passband_gains.len() / num_fine_chans_per_coarse
+    } else {
+        return Err(BirliError::BadArrayShape {
+            argument: "passband_gains".into(),
+            function: "correct_coarse_passband_gains".into(),
+            expected: format!(
+                "n, where n is a multiple of num_fine_chans_per_coarse={}",
+                num_fine_chans_per_coarse
+            ),
+            received: format!("{:?}", passband_gains.len()),
+        });
+    };
+
+    let scrunched_gains = scrunch_gains(passband_gains, fscrunch, mwax);
+
+    for (mut jones_array, mut weight_array) in izip!(
+        jones_array.axis_chunks_iter_mut(Axis(1), num_fine_chans_per_coarse),
+        weight_array.axis_chunks_iter_mut(Axis(1), num_fine_chans_per_coarse),
+    ) {
+        for (mut jones_array, mut weight_array, &gain) in izip!(
+            jones_array.axis_iter_mut(Axis(1)),
+            weight_array.axis_iter_mut(Axis(1)),
+            scrunched_gains.iter()
+        ) {
+            for (jones, weight) in izip!(jones_array.iter_mut(), weight_array.iter_mut()) {
+                let jones_f64 = Jones::<f64>::from(*jones);
+                *jones = Jones::<f32>::from(jones_f64 / gain);
+                *weight = (gain * (*weight as f64)) as f32;
+            }
+        }
+    }
+
+    Ok(())
+}
+
+/// Given a set of gains for each ultrafine channel, compute the gains for each fine channel,
+/// is if these were averaged by given correlator type.
+///
+/// # Averaging
+///
+/// a scrunched channel contains fscrunch times the bandwidth of the ultrafine channel.
+///
+/// MWAX example with 12 ultrafine channels:
+/// fscrunch|0| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |0|
+/// --------|-|---|---|---|---|---|---|---|---|---|---|---|-|
+/// 2 (e->e)| 0*|   1   |   2   |   3   |   4   |   5   | 0*|
+/// 3 (o->e)|  0* |     1     |     2     |     3     |  0* |
+/// 4 (e->o)|       0       |       1       |       2       |
+///
+/// MWAX example with 15 ultrafine channels:
+/// fscrunch|0| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 |10 |11 |12 |13 |14 |0|
+/// --------|-|---|---|---|---|---|---|---|---|---|---|---|---|---|---| |
+/// 3 (o->o)|     0*    |     1     |     2     |     3     |     4     |
+/// 5 (o->o)|           0       |         1         |         2         |
+///
+/// For more details see: https://wiki.mwatelescope.org/display/MP/MWA+Fine+Channel+Centre+Frequencies
+pub fn scrunch_gains(ultrafine_gains: &[f64], fscrunch: usize, mwax: bool) -> Vec<f64> {
+    let scrunched_length = ultrafine_gains.len() / fscrunch;
+    if fscrunch == 1 {
+        ultrafine_gains.to_vec()
+    } else if !mwax {
+        ultrafine_gains
+            .chunks(fscrunch)
+            .map(|gains_chunk| gains_chunk.iter().sum::<f64>() / (fscrunch as f64))
+            .collect()
+    } else {
+        // in mwax, for an even number of scrunched channels, a given scrunched channel `n`:
+        // - is centered on ultrafine channel `n * fscrunch`.
+        // - contains power from the center channel, and some power from channels within
+        //   `(fscrunch - 1) / 2` channels of the centre, wrapping around the coarse channel
+        //
+        // Scrunched channel zero will contain an equal amount power from the low and high ends of
+        // the coarse channel.
+
+        // in mwax, for an odd number of scrunched channels, a given scrunched channel `n`:
+        // - contains half the power of ultrafine channel `n * fscrunch`.
+        // - contains all of the power from ultrafine channels `n * fscrunch + (1..n-1)`
+        // - contains half the power from ultrafine channel `n * fscrunch + n`
+        // - centered on ultrafine channel `(n + 1/2) * fscrunch`.s
+        //
+        // Scrunched channel zero will contain an equal amount power from the low and high ends of
+        // the coarse channel.
+        // Here we use a cycle iterator, and calculate the first ultrafine channel which contributes
+        // to scrunched channel zero, `first_zeroth_channel`.
+
+        // get the relative index and weight of the window of ultrafine channels that contribute to a scrunched channel.
+        #[rustfmt::skip]
+        let window_offset_weights: Vec<(i32, f64)> = match (scrunched_length % 2, fscrunch % 2) {
+            // even channels, even fscrunch: window length is fscrunch + 1, half-weighted edges
+            (0, 0) => (0..fscrunch + 1)
+                .map(|w| (
+                    (w as i32 - fscrunch as i32 / 2),
+                    (if w == 0 || w == fscrunch { 0.5 } else { 1. }) / fscrunch as f64,
+                ))
+                .collect(),
+            // even channels, odd fscrunch: window length is fscrunch, equal weights
+            (0, 1) => (0..fscrunch)
+                .map(|w| (
+                    (w as i32 - (fscrunch as i32 - 1) / 2),
+                    1. / fscrunch as f64
+                ))
+                .collect(),
+            // odd channels: window length is fscrunch + 1, half-weighted edges
+            (1, _) => (0..fscrunch + 1)
+                .map(|w| (
+                    (w as i32),
+                    (if w == 0 || w == fscrunch { 0.5 } else { 1. }) / fscrunch as f64,
+                ))
+                .collect(),
+            // odd channels, odd fscrunch: window length is fscrunch + 1, half-weighted edges
+            // (1, 1) => (0..fscrunch + 1).map(|w| (
+            //     (w as i32),
+            //     (if w == 0 || w == fscrunch { 0.5 } else { 1. }) / fscrunch as f64
+            // )).collect(),
+            (_, _) => unreachable!(),
+        };
+        if scrunched_length % 2 == 0 {
+            // sanity check: centre symmetric indices should sum to zero, weights should sum to one
+            assert_eq!(
+                window_offset_weights
+                    .iter()
+                    .fold((0, 0.), |(sum_o, sum_w), (o, w)| { (sum_o + o, sum_w + w) }),
+                (0, 1.)
+            );
+        }
+        // apply the weights to calculate the scrunched gains
+        (0..scrunched_length)
+            .map(|scrunched_chan| {
+                window_offset_weights
+                    .iter()
+                    .fold(0., |acc, &(offset, weight)| {
+                        // rem_euclid is basically mod but it correctly wraps around with negative numbers.
+                        let ultrafine_chan = (((fscrunch * scrunched_chan) as i32 + offset)
+                            .rem_euclid(ultrafine_gains.len() as i32))
+                            as usize;
+                        acc + ultrafine_gains[ultrafine_chan] * weight
+                    })
+            })
+            .collect()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     #![allow(clippy::float_cmp)]
 
-    use super::{correct_cable_lengths, correct_digital_gains, correct_geometry, VEL_C};
+    use super::{
+        _correct_digital_gains, correct_cable_lengths, correct_coarse_passband_gains,
+        correct_digital_gains, correct_geometry, scrunch_gains, VEL_C,
+    };
+    use itertools::izip;
     use marlu::{
         hifitime::Epoch, mwalib::CorrelatorContext, precession::precess_time, Complex, Jones,
         LatLngHeight, RADec, XyzGeodetic, UVW,
     };
-    use ndarray::{s, Array2, Array3};
+    use ndarray::{s, Array2, Array3, Axis};
     use std::f64::consts::PI;
 
     use crate::{
         approx::assert_abs_diff_eq,
         context_to_jones_array,
-        corrections::_correct_digital_gains,
         flags::{get_coarse_chan_range, get_timestep_range},
         get_flaggable_timesteps, BirliError, TestJones,
     };
@@ -1378,4 +1571,247 @@ mod tests {
             Err(BirliError::BadArrayShape { .. })
         ));
     }
+
+    #[test]
+    fn test_scrunch_gains_legacy() {
+        let base: i32 = 2;
+        let ultrafine_gains: Vec<f64> = (0..30).map(|x| (base.pow(x)) as _).collect();
+        let expected_gains: Vec<f64> = (0..15)
+            .map(|x| ((base.pow(2 * x)) + (base.pow(2 * x + 1))) as f64 / 2.)
+            .collect();
+        let scrunched_gains = scrunch_gains(&ultrafine_gains, 2, false);
+        assert_eq!(scrunched_gains, expected_gains);
+    }
+
+    #[test]
+    fn test_scrunch_gains_mwax_even_scrunch_even_channels() {
+        let base: i32 = 2;
+        let ultrafine_gains: Vec<f64> = (0..12).map(|x| (base.pow(x)) as _).collect();
+        let expected_gains: Vec<f64> = (0..12 / 2)
+            .map(|x| -> f64 {
+                let left = ultrafine_gains
+                    [(2 * x as i32 - 1).rem_euclid(ultrafine_gains.len() as i32) as usize]
+                    as f64;
+                let center = ultrafine_gains[2 * x as usize] as f64;
+                let right = ultrafine_gains[(2 * x + 1) as usize] as f64;
+                (left / 4. + center / 2. + right / 4.) as f64
+            })
+            .collect();
+        let scrunched_gains = scrunch_gains(&ultrafine_gains, 2, true);
+        assert_eq!(scrunched_gains, expected_gains);
+    }
+
+    #[test]
+    fn test_scrunch_gains_mwax_odd_scrunch_even_channels() {
+        let base: i32 = 2;
+        let ultrafine_gains: Vec<f64> = (0..12).map(|x| (base.pow(x)) as _).collect();
+        let expected_gains: Vec<f64> = (0..12 / 3)
+            .map(|x| -> f64 {
+                let left = ultrafine_gains
+                    [(3 * x as i32 - 1).rem_euclid(ultrafine_gains.len() as i32) as usize]
+                    as f64;
+                let center = ultrafine_gains[3 * x] as f64;
+                let right = ultrafine_gains[3 * x + 1] as f64;
+                (left / 3. + center / 3. + right / 3.) as f64
+            })
+            .collect();
+        let scrunched_gains = scrunch_gains(&ultrafine_gains, 3, true);
+        assert_eq!(scrunched_gains, expected_gains);
+    }
+
+    #[test]
+    fn test_scrunch_gains_mwax_even_scrunch_odd_channels() {
+        let base: i32 = 2;
+        let ultrafine_gains: Vec<f64> = (0..12).map(|x| (base.pow(x)) as _).collect();
+        let expected_gains: Vec<f64> = (0..12 / 4)
+            .map(|x| -> f64 {
+                let left = ultrafine_gains[4 * x] as f64;
+                let center1 = ultrafine_gains[4 * x + 1] as f64;
+                let center2 = ultrafine_gains[4 * x + 2] as f64;
+                let center3 = ultrafine_gains[4 * x + 3] as f64;
+                let right = ultrafine_gains
+                    [(4 * x as i32 + 4).rem_euclid(ultrafine_gains.len() as i32) as usize]
+                    as f64;
+                (left / 8. + center1 / 4. + center2 / 4. + center3 / 4. + right / 8.) as f64
+            })
+            .collect();
+        let scrunched_gains = scrunch_gains(&ultrafine_gains, 4, true);
+        assert_eq!(scrunched_gains, expected_gains);
+    }
+
+    #[test]
+    fn test_scrunch_gains_mwax_odd_scrunch_odd_channels() {
+        let base: i32 = 2;
+        let ultrafine_gains: Vec<f64> = (0..15).map(|x| (base.pow(x)) as _).collect();
+        let expected_gains: Vec<f64> = (0..15 / 3)
+            .map(|x| -> f64 {
+                let left = ultrafine_gains[3 * x] as f64;
+                let center1 = ultrafine_gains[3 * x + 1] as f64;
+                let center2 = ultrafine_gains[3 * x + 2] as f64;
+                let right = ultrafine_gains
+                    [(3 * x as i32 + 3).rem_euclid(ultrafine_gains.len() as i32) as usize]
+                    as f64;
+                (left / 6. + center1 / 3. + center2 / 3. + right / 6.) as f64
+            })
+            .collect();
+        let scrunched_gains = scrunch_gains(&ultrafine_gains, 3, true);
+        assert_eq!(scrunched_gains, expected_gains);
+    }
+
+    #[test]
+    fn test_correct_coarse_passband_gains() {
+        let num_fine_chans_per_coarse = 2;
+        // 2 coarse channels
+        let mut jones_array: Array3<Jones<f32>> =
+            Array3::from_shape_fn((2, 4, 2), |_| Jones::identity());
+        let mut weight_array: Array3<f32> = Array3::from_shape_fn((2, 4, 2), |_| 1.0);
+        let passband_gains = vec![0.1, 0.2];
+
+        correct_coarse_passband_gains(
+            &mut jones_array,
+            &mut weight_array,
+            &passband_gains,
+            num_fine_chans_per_coarse,
+            false,
+        )
+        .unwrap();
+
+        let exp_gains = [0.1, 0.2, 0.1, 0.2];
+
+        for (&exp_gain, jones_array, weight_array) in izip!(
+            exp_gains.iter(),
+            jones_array.axis_iter(Axis(1)),
+            weight_array.axis_iter(Axis(1))
+        ) {
+            for (&jones, &weight) in izip!(jones_array.iter(), weight_array.iter()) {
+                compare_jones!(Jones::identity() / exp_gain, jones);
+                assert_eq!(exp_gain, weight);
+            }
+        }
+    }
+
+    #[test]
+    fn test_correct_coarse_passband_gains_good_fscrunch() {
+        let num_fine_chans_per_coarse = 2;
+        // 2 coarse channels
+        let mut jones_array: Array3<Jones<f32>> =
+            Array3::from_shape_fn((2, 4, 2), |_| Jones::identity());
+        let mut weight_array: Array3<f32> = Array3::from_shape_fn((2, 4, 2), |_| 1.0);
+        let passband_gains = vec![0.1, 0.2, 0.3, 0.4];
+
+        correct_coarse_passband_gains(
+            &mut jones_array,
+            &mut weight_array,
+            &passband_gains,
+            num_fine_chans_per_coarse,
+            false,
+        )
+        .unwrap();
+
+        // TODO: actually test weights
+
+        let exp_gains = [0.15, 0.35, 0.15, 0.35];
+
+        for (&exp_gain, jones_array, weight_array) in izip!(
+            exp_gains.iter(),
+            jones_array.axis_iter(Axis(1)),
+            weight_array.axis_iter(Axis(1))
+        ) {
+            for (&jones, &weight) in izip!(jones_array.iter(), weight_array.iter()) {
+                compare_jones!(Jones::identity() / exp_gain, jones);
+                assert_eq!(exp_gain, weight);
+            }
+        }
+    }
+
+    #[test]
+    fn test_correct_coarse_passband_gains_high_fscrunch() {
+        let num_fine_chans_per_coarse = 2;
+        // 2 coarse channels
+        let mut jones_array: Array3<Jones<f32>> =
+            Array3::from_shape_fn((2, 4, 2), |_| Jones::identity());
+        let mut weight_array: Array3<f32> = Array3::from_shape_fn((2, 4, 2), |_| 1.0);
+        let passband_gains = vec![0.1, 0.2, 0.3, 100., 0.5, 0.6, 0.7, 100.];
+
+        correct_coarse_passband_gains(
+            &mut jones_array,
+            &mut weight_array,
+            &passband_gains,
+            num_fine_chans_per_coarse,
+            false,
+        )
+        .unwrap();
+
+        let exp_gains = [25.15, 25.45, 25.15, 25.45];
+
+        for (&exp_gain, jones_array, weight_array) in izip!(
+            exp_gains.iter(),
+            jones_array.axis_iter(Axis(1)),
+            weight_array.axis_iter(Axis(1))
+        ) {
+            for (&jones, &weight) in izip!(jones_array.iter(), weight_array.iter()) {
+                compare_jones!(Jones::identity() / exp_gain, jones);
+                assert_eq!(exp_gain, weight);
+            }
+        }
+    }
+
+    #[test]
+    fn test_correct_coarse_passband_gains_bad_array_shape() {
+        let num_fine_chans_per_coarse = 2;
+        let mut jones_array: Array3<Jones<f32>> =
+            Array3::from_shape_fn((2, 4, 2), |_| Jones::identity());
+        let mut weight_array: Array3<f32> = Array3::from_shape_fn((2, 4, 2), |_| 1.0);
+        let passband_gains = vec![0.1, 0.2, 0.3, 100., 0.5, 0.6, 0.7, 100.];
+
+        // test num_fine_chans_per_coarse=0
+        assert!(matches!(
+            correct_coarse_passband_gains(
+                &mut jones_array,
+                &mut weight_array,
+                &passband_gains,
+                0,
+                false,
+            ),
+            Err(BirliError::BadArrayShape { .. })
+        ));
+
+        // test bad jones array shape
+        assert!(matches!(
+            correct_coarse_passband_gains(
+                &mut jones_array,
+                &mut weight_array,
+                &passband_gains,
+                3,
+                false,
+            ),
+            Err(BirliError::BadArrayShape { .. })
+        ));
+
+        // test weight_array dimension mismatch with jones_array
+        let mut bad_weight_array: Array3<f32> = Array3::from_shape_fn((2, 4, 3), |_| 1.0);
+        assert!(matches!(
+            correct_coarse_passband_gains(
+                &mut jones_array,
+                &mut bad_weight_array,
+                &passband_gains,
+                num_fine_chans_per_coarse,
+                false,
+            ),
+            Err(BirliError::BadArrayShape { .. })
+        ));
+
+        // test bad gain shape
+        let bad_passband_gains = vec![0.1, 0.2, 0.3];
+        assert!(matches!(
+            correct_coarse_passband_gains(
+                &mut jones_array,
+                &mut weight_array,
+                &bad_passband_gains,
+                num_fine_chans_per_coarse,
+                false,
+            ),
+            Err(BirliError::BadArrayShape { .. })
+        ));
+    }
 }
